import UIKit

// This is a subclass of the layout from the Custom Layout page
public class AutosizingBoringLayout: BoringLayout {
    
    var heightCache = [Int: CGFloat]()
    
    override public func frame(for indexPath: IndexPath) -> CGRect {
        // Now we can't use a constant height for each frame, so must calculate from the cache
        let heightsTillNow: CGFloat = (0..<indexPath.item).reduce(0) {
            return $0 + self.spacing + (self.heightCache[$1] ?? self.itemHeight)
        }
        
        return CGRect(x: 0, y: heightsTillNow, width: self.width, height: self.heightCache[indexPath.item] ?? self.itemHeight)
    }
    
    override public func shouldInvalidateLayout(forPreferredLayoutAttributes preferredAttributes: UICollectionViewLayoutAttributes, withOriginalAttributes originalAttributes: UICollectionViewLayoutAttributes) -> Bool {
        
        print(#function)
        // Do we have a recorded size for this item?
        let index = originalAttributes.indexPath.item
        if let height = self.heightCache[index], height == preferredAttributes.size.height {
            return false
        } else {
            return true
        }
    }
    
    override public func invalidationContext(forPreferredLayoutAttributes preferredAttributes: UICollectionViewLayoutAttributes, withOriginalAttributes originalAttributes: UICollectionViewLayoutAttributes) -> UICollectionViewLayoutInvalidationContext {
        print(#function)
        let context = super.invalidationContext(forPreferredLayoutAttributes: preferredAttributes, withOriginalAttributes: originalAttributes)
        
        let oldContentSize = self.collectionViewContentSize
        self.heightCache[originalAttributes.indexPath.item] = preferredAttributes.size.height
        let newContentSize = self.collectionViewContentSize
        context.contentSizeAdjustment = CGSize(width: 0, height: newContentSize.height - oldContentSize.height)
        // If an item has changed size, the layout of everything underneath it has also been invalidated.
        let indexPaths: [IndexPath] = (originalAttributes.indexPath.item..<self.numberOfItems).map {
            return IndexPath(item: $0, section: 0)
        }
        context.invalidateItems(at: indexPaths)
        
        return context
    }
    
}

