//: ### A Custom Layout
import PlaygroundSupport
import UIKit

//: Open the console to see method call logging. 
class BoringLayout: UICollectionViewLayout {
    
    var itemHeight: CGFloat = 50
    var spacing: CGFloat = 10
    private var width: CGFloat = 0
    private var numberOfItems = 0
    
    // Prepare is called before any of the layout is calculated
    override func prepare() {
        print(#function)
        super.prepare()
        self.width = self.collectionView?.bounds.width ?? 0
        self.numberOfItems = self.collectionView?.numberOfItems(inSection: 0) ?? 0
    }
    
    override var collectionViewContentSize: CGSize {
        print(#function)
        // This layout only supports one section
        
        let height = CGFloat(self.numberOfItems) * (self.itemHeight + self.spacing)
        return CGSize(width: self.width, height: height)
    }
    
    // This function is called periodically, using rects that the collection view is asking for.
    override func layoutAttributesForElements(in rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        print(#function)
        // You could optimise here by not checking every single index path in the collection view, for example
        let attributes: [UICollectionViewLayoutAttributes] = (0..<self.numberOfItems).flatMap {
            
            let indexPath = IndexPath(item: $0, section: 0)
            
            let frame = self.frame(for: indexPath)
            if !frame.intersects(rect) {
                return nil
            }
            
            return self.layoutAttributesForItem(at: indexPath)
            
        }
        
        return attributes
    }
    
    private func frame(for indexPath: IndexPath) -> CGRect {
        return CGRect(
            x: 0,
            y: CGFloat(indexPath.item) * (self.itemHeight + self.spacing),
        width: self.collectionView?.bounds.width ?? 0,
        height: self.itemHeight)
    }
    
    override func layoutAttributesForItem(at indexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        print(#function)
        let attributes = UICollectionViewLayoutAttributes(forCellWith: indexPath)
        attributes.frame = self.frame(for: indexPath)
        return attributes
    }
}

let layout = BoringLayout()

let collectionView = UICollectionView(frame: CGRect(x: 0, y: 0, width: 375, height: 667), collectionViewLayout: layout)
collectionView.register(TextCell.self, forCellWithReuseIdentifier: "cell")

let datasource = DataSource()
collectionView.dataSource = datasource
PlaygroundPage.current.liveView = collectionView

//: [Next](@next)
//:
//: [Previous](@previous)

