//: ### Decoration Views
import PlaygroundSupport
import UIKit

//: This is a very simple view class which will be the decoration view
class Decoration: UICollectionReusableView {
    override func apply(_ layoutAttributes: UICollectionViewLayoutAttributes) {
        super.apply(layoutAttributes)
        self.backgroundColor = .blue
        self.layer.borderWidth = 2
        self.layer.borderColor = UIColor.black.cgColor
    }
}

//: This is a subclass of the layout from the Custom Layout page
class DecoratedBoringLayout: BoringLayout {
    
    let decorationKind = "decoration"
    var decorationFrame = CGRect(x: 10, y: 10, width: 20, height: 20)
    
    override init() {
        super.init()
        self.register(Decoration.self, forDecorationViewOfKind: decorationKind)
    }
    
    override func prepare() {
        super.prepare()
        self.decorationFrame.origin.x = self.width - 20 - self.decorationFrame.width
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func layoutAttributesForElements(in rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        var attributes = super.layoutAttributesForElements(in: rect)
        // There is a single decoration view
        if rect.intersects(self.decorationFrame), let decorationAttributes = self.layoutAttributesForDecorationView(ofKind: decorationKind, at: IndexPath(item: 0, section: 0)) {
            attributes?.append(decorationAttributes)
        }
        return attributes
    }
    
    override func layoutAttributesForDecorationView(ofKind elementKind: String, at indexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        print(#function)
        let attributes = UICollectionViewLayoutAttributes(forDecorationViewOfKind: elementKind, with: indexPath)
        attributes.frame = decorationFrame
        return attributes
    }
}

let layout = DecoratedBoringLayout()

let collectionView = UICollectionView(frame: CGRect(x: 0, y: 0, width: 375, height: 667), collectionViewLayout: layout)
collectionView.register(TextCell.self, forCellWithReuseIdentifier: "cell")

let datasource = DataSource()
collectionView.dataSource = datasource
PlaygroundPage.current.liveView = collectionView

//: [Next](@next)
//:
//: [Previous](@previous)

