//: ### Live views for quick iteration of collection view layout development
import UIKit
import PlaygroundSupport

//: Create a simple collection view layout
let layout = UICollectionViewFlowLayout()
layout.itemSize = CGSize(width: 50, height: 50)

//: Create a collection view using the layout
let view = UICollectionView(frame: CGRect(x: 0, y: 0, width: 375, height: 667), collectionViewLayout: layout)

//: For this demo, we're not using a specific cell subclass
view.register(UICollectionViewCell.self, forCellWithReuseIdentifier: "cell")

//: This object is a collection view datasource that will return colored cells
class DataSource: NSObject, UICollectionViewDataSource {
    
    private let totalCells = 150
    
    func numberOfSections(in collectionView: UICollectionView) -> Int {
        return 1
    }
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return totalCells
    }
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "cell", for: indexPath)
        cell.backgroundColor = UIColor(hue: CGFloat(indexPath.item) / 255, saturation: 1, brightness: 1, alpha: 1)
        return cell
    }
}

let dataSource = DataSource()
view.dataSource = dataSource
//: You could add this view inline, but it wouldn't be interactive. Set it as the live view and open the assistant editor, and you can scroll it. If nothing shows up in the Assistant editor, try quitting the iOS simulator and Xcode and running again :( 
PlaygroundPage.current.liveView = view

//: [Next](@next)
