import UIKit

open class BoringLayout: UICollectionViewLayout {
    
    public var itemHeight: CGFloat = 50
    public var spacing: CGFloat = 10
    public var width: CGFloat = 0
    public var numberOfItems = 0
    
    // Prepare is called before any of the layout is calculated
    override open func prepare() {
        print(#function)
        super.prepare()
        self.width = self.collectionView?.bounds.width ?? 0
        self.numberOfItems = self.collectionView?.numberOfItems(inSection: 0) ?? 0
    }
    
    override open var collectionViewContentSize: CGSize {
        print(#function)
        // This layout only supports one section
        
        let height = self.frame(for: IndexPath(item: self.numberOfItems, section: 0)).maxY
        return CGSize(width: self.width, height: height)
    }
    
    // This function is called periodically, using rects that the collection view is asking for.
    override open func layoutAttributesForElements(in rect: CGRect) -> [UICollectionViewLayoutAttributes]? {
        print(#function)
        // You could optimise here by not checking every single index path in the collection view, for example
        let attributes: [UICollectionViewLayoutAttributes] = (0..<self.numberOfItems).flatMap {
            
            let indexPath = IndexPath(item: $0, section: 0)
            
            let frame = self.frame(for: indexPath)
            if !frame.intersects(rect) {
                return nil
            }
            
            return self.layoutAttributesForItem(at: indexPath)
            
        }
        
        return attributes
    }
    
    open func frame(for indexPath: IndexPath) -> CGRect {
        return CGRect(
            x: 0,
            y: CGFloat(indexPath.item) * (self.itemHeight + self.spacing),
            width: self.collectionView?.bounds.width ?? 0,
            height: self.itemHeight)
    }
    
    override open func layoutAttributesForItem(at indexPath: IndexPath) -> UICollectionViewLayoutAttributes? {
        print(#function)
        let attributes = UICollectionViewLayoutAttributes(forCellWith: indexPath)
        attributes.frame = self.frame(for: indexPath)
        return attributes
    }
}


